//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.AddCustomGetHandler)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Linq;
using System.Text;
//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
using System.Web.UI;
//!<# } #>
//!<# if (Params.CalDav || Params.CardDav) { #>
using System.Text.RegularExpressions;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Class1;
using ITHit.WebDAV.Server.Extensibility;
//!<# if (Params.CalDav) { #>
using ITHit.WebDAV.Server.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDAV.Server.CardDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IMethodHandlerAsync
    /// <summary>
    /// This handler processes GET and HEAD requests to folders returning custom HTML page.
    /// </summary>
    internal class MyCustomGetHandler : IMethodHandlerAsync
    {
        /// <summary>
        /// Handler for GET and HEAD request registered with the engine before registering this one.
        /// We call this default handler to handle GET and HEAD for files, because this handler
        /// only handles GET and HEAD for folders.
        /// </summary>
        public IMethodHandlerAsync OriginalHandler { get; set; }

        /// <summary>
        /// Gets a value indicating whether output shall be buffered to calculate content length.
        /// Don't buffer output to calculate content length.
        /// </summary>
        public bool EnableOutputBuffering
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether engine shall log response data (even if debug logging is on).
        /// </summary>
        public bool EnableOutputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether the engine shall log request data.
        /// </summary>
        public bool EnableInputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Handles GET and HEAD request.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContextBaseAsync"/>.</param>
        /// <param name="item">Instance of <see cref="IHierarchyItemAsync"/> which was returned by
        /// <see cref="DavContextBaseAsync.GetHierarchyItemAsync"/> for this request.</param>
        public async Task ProcessRequestAsync(DavContextBaseAsync context, IHierarchyItemAsync item)
        {
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            string exePath = Path.GetDirectoryName(new Uri(Assembly.GetExecutingAssembly().CodeBase).LocalPath);
            string htmlPath = Directory.GetParent(exePath).FullName;
            //!<# } #>
            if (item is IItemCollectionAsync)
            {
                // In case of GET requests to WebDAV folders we serve a web page to display 
                // any information about this server and how to use it.

                // Remember to call EnsureBeforeResponseWasCalledAsync here if your context implementation
                // makes some useful things in BeforeResponseAsync.
                await context.EnsureBeforeResponseWasCalledAsync();

                //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
                IHttpAsyncHandler page = (IHttpAsyncHandler)System.Web.Compilation.BuildManager.CreateInstanceFromVirtualPath(
                    "~/MyCustomHandlerPage.aspx", typeof(MyCustomHandlerPage));
                //!<# if (Params.TargetFramework45OrLater) { #>

                // Here we call BeginProcessRequest instead of ProcessRequest to start an async page execution and be able to call RegisterAsyncTask if required. 
                // To call APM method (Begin/End) from TAP method (Task/async/await) the Task.FromAsync must be used.
                await Task.Factory.FromAsync(page.BeginProcessRequest, page.EndProcessRequest, HttpContext.Current, null);
                //!<# } else { #>
                page.ProcessRequest(HttpContext.Current);
                //!<# } #>
                //!<# } #>

                //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
                using (TextReader reader = new StreamReader(Path.Combine(htmlPath, "MyCustomHandlerPage.html")))
                {
                    string html = await reader.ReadToEndAsync();
                    //!<# if (Params.CalDav || Params.CardDav) { #>
                    html = html.Replace("_webDavServerUrl_", context.Request.UrlPrefix + context.Request.ApplicationPath);
                    //!<# } #>
                    //!<# if (Params.CalDav) { #>
                    html = html.Replace("_CALENDARS_", await AllUserCalendarsUrlAsync(context));
                    //!<# } #>
                    //!<# if (Params.CardDav) { #>
                    html = html.Replace("_BOOKS_", await AllUserAddressbooksUrlAsync(context));
                    //!<# } #>
                    html = html.Replace("_webDavServerRoot_", context.Request.ApplicationPath.TrimEnd('/'));
                    html = html.Replace("_webDavServerVersion_",
                        Assembly.GetAssembly(typeof(DavEngineAsync)).GetName().Version.ToString());

                    await WriteHtmlAsync(context, html);
                }
                //!<# } #>
            }
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            else if (context.Request.RawUrl.StartsWith("/AjaxFileBrowser/"))
            {
                // The "/AjaxFileBrowser/" is not a WebDAV folder. It can be used to store client script files, 
                // images, static HTML files or any other files that does not require access via WebDAV.
                // Any request to the files in this folder will just serve them to client. 

                await context.EnsureBeforeResponseWasCalledAsync();
                string filePath = Path.Combine(htmlPath, context.Request.RawUrl.TrimStart('/').Replace("/", "\\"));

                // Remove query string.
                int queryIndex = filePath.LastIndexOf('?');
                if (queryIndex > -1)
                {
                    filePath = filePath.Remove(queryIndex);
                }

                if (!File.Exists(filePath))
                {
                    throw new DavException("File not found: " + filePath, DavStatus.NOT_FOUND);
                }

                using (TextReader reader = new StreamReader(filePath))
                {
                    string html = await reader.ReadToEndAsync();
                    await WriteHtmlAsync(context, html);
                }
            }
            //!<# } #>
            else
            {
                await OriginalHandler.ProcessRequestAsync(context, item);
            }
        }

        //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
        /// <summary>
        /// Writes HTML to the output stream in case of GET request using encoding specified in Engine. 
        /// Writes headers only in caes of HEAD request.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContextBaseAsync"/>.</param>
        /// <param name="html">HTML to write.</param>
        private async Task WriteHtmlAsync(DavContextBaseAsync context, string html)
        {
            Encoding encoding = context.Engine.ContentEncoding; // UTF-8 by default
            context.Response.ContentLength = encoding.GetByteCount(html);
            context.Response.ContentType = string.Format("text/html; charset={0}", encoding.WebName);

            // Return file content in case of GET request, in case of HEAD just return headers.
            if (context.Request.HttpMethod == "GET")
            {
                using (var writer = new StreamWriter(context.Response.OutputStream, encoding))
                {
                    await writer.WriteAsync(html);
                }
            }
        }
        //!<# } #>

        /// <summary>
        /// This handler shall only be invoked for <see cref="IFolderAsync"/> items or if original handler (which
        /// this handler substitutes) shall be called for the item.
        /// </summary>
        /// <param name="item">Instance of <see cref="IHierarchyItemAsync"/> which was returned by
        /// <see cref="DavContextBaseAsync.GetHierarchyItemAsync"/> for this request.</param>
        /// <returns>Returns <c>true</c> if this handler can handler this item.</returns>
        public bool AppliesTo(IHierarchyItemAsync item)
        {
            return item is IFolderAsync || OriginalHandler.AppliesTo(item);
        }

        //!<# if (Params.Hosting == HostingEnvironment.Listener && Params.CalDav) { #>
        /// <summary>
        /// Gets all user calendars URLs.
        /// </summary>
        private async Task<string> AllUserCalendarsUrlAsync(DavContextBaseAsync context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<IHierarchyItemAsync> items = new List<IHierarchyItemAsync>();

            // get list of folders that contain user calendars and enumerate calendars in each folder
            foreach (IItemCollectionAsync folder in await discovery.GetCalendarHomeSetAsync())
            {
                IEnumerable<IHierarchyItemAsync> children = await folder.GetChildrenAsync(new PropertyName[0]);
                items.AddRange(children.Where(x => x is ICalendarFolderAsync));
            }
            IEnumerable<string> nameAndUrls = items.Select(x => string.Format("{0}: {1}", x.Name, AddAppPath(context, x.Path)));
            return String.Join("<br>", nameAndUrls.ToArray());
        }
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.Listener && Params.CardDav) { #>
        /// <summary>
        /// Gets all user address books URLs.
        /// </summary>
        private async Task<string> AllUserAddressbooksUrlAsync(DavContextBaseAsync context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<IHierarchyItemAsync> items = new List<IHierarchyItemAsync>();

            // get list of folders that contain user address books and enumerate address books in each folder
            foreach (IItemCollectionAsync folder in await discovery.GetAddressbookHomeSetAsync())
            {
                IEnumerable<IHierarchyItemAsync> children = await folder.GetChildrenAsync(new PropertyName[0]);
                items.AddRange(children.Where(x => x is IAddressbookFolderAsync));
            }
            IEnumerable<string> nameAndUrls = items.Select(x => string.Format("{0}: {1}", x.Name, AddAppPath(context, x.Path)));
            return String.Join("<br>", nameAndUrls.ToArray());
        }
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.Listener && (Params.CalDav || Params.CardDav)) { #>


        private static string AddAppPath(DavContextBaseAsync context, string path)
        {
            string applicationPath = context.Request.UrlPrefix + context.Request.ApplicationPath;
            return string.Format("{0}/{1}", applicationPath.TrimEnd(new[] { '/' }), path.TrimStart(new[] { '/' }));
        }
        //!<# } #>
    }
    //$>
}
