//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>


namespace ITHit.WebDav.Server.SampleModel.__AclSimpl__.Acl
{
    /// <summary>
    /// Logical folder with name 'acl' which contains folders 'users' and 'groups'.
    /// Instances of this class correspond to the following path: [DAVLocation]/acl/
    /// </summary>
    public class AclFolder : LogicalFolder
    {
        /// <summary>
        /// This folder name.
        /// </summary>
        private static readonly string aclFolderName = "acl";

        /// <summary>
        /// Path to this folder.
        /// </summary>
        public static readonly string AclFolderPath = string.Format("{0}{1}/", DavLocationFolder.DavLocationFolderPath, aclFolderName);

        //!<# if (false) {#>
        //TODO: check read privilege in all property gets?
        //!<# } #>
        /// <summary>
        /// Initializes a new instance of the AclFolder class.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContext"/>.</param>
        public AclFolder(DavContext context) : base(context, aclFolderName, AclFolderPath)
        {
        }

        /// <summary>
        /// Retrieves children of /acl folder.
        /// We have here 'user' and 'group' folder for holding users and groups respectively.
        /// </summary>
        /// <param name="propNames">Property names to be fetched lated.</param>
        /// <returns>Children of this folder.</returns>
        public override async Task<IEnumerable<IHierarchyItemAsync>> GetChildrenAsync(IList<PropertyName> propNames)
        {
            return new[] {new UsersFolder(Context)};
        }
    }
}
