//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Blank)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;

namespace ITHit.WebDav.Server.SampleModel.__DavImpl__
{
    /// <summary>
    /// Folder named 'acl' in the root of the repository.
    /// It contains folders 'users' and 'groups'
    /// </summary>
    public class AclFolder : LogicalFolder
    {
        /// <summary>
        /// Initializes a new instance of the AclFolder class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/> class.</param>
        public AclFolder(DavContext context) : base(context, "acl", "acl/")
        {
        }

        /// <summary>
        /// Returns child folders inside '/acl/' folder.
        /// For example two folders 'users' and 'groups'.
        /// </summary>
        /// <param name="propNames">Properties that will be retrieved lated from the item returned by the engine.
        /// </param>
        /// <returns>List of child items.</returns>
        public override async Task<IEnumerable<IHierarchyItemAsync>> GetChildrenAsync(IList<PropertyName> propNames)
        {
            yield return new UserFolder(Context);
            yield return new GroupFolder(Context);
        }
    }
}
