//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
//!<# if (Params.ACL) { #>
using System.Security.AccessControl;
using System.Security.Principal;
//!<# } #>
//!<# if (Params.IsOwinAuthMode) { #>
using Microsoft.AspNet.Identity;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
//!<# if (Params.Class2) { #>
using ITHit.WebDAV.Server.Class2;
//!<# } #>
//!<# if (Params.MicrosoftExtensions) { #>
using ITHit.WebDAV.Server.MicrosoftExtensions;
//!<# } #>
//!<# if (Params.ACL) { #>

using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl;
//!<# } #>
//!<# if (Params.ACL && !Params.TargetFramework45OrLater) { #>
using IPrincipal = ITHit.WebDAV.Server.Acl.IPrincipalAsync;
//!<# } #>


namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// Base class for WebDAV items (folders, files, etc).
    /// </summary>
    public abstract class DavHierarchyItem :
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ Discovery/*!<# } #>!*/
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ ,/*!<# } #>!*/ IHierarchyItemAsync
        /*!<# if (Params.ACL && !(Params.CalDav || Params.CardDav)) { #>!*/, IAclHierarchyItemAsync/*!<# } #>!*/
        /*!<# if (Params.ACL &&  (Params.CalDav || Params.CardDav)) { #>!*/, ICurrentUserPrincipalAsync/*!<# } #>!*/
        /*!<# if (Params.Class2) { #>!*/, ILockAsync/*!<# } #>!*/
        /*!<# if (Params.MicrosoftExtensions) { #>!*/, IMsItemAsync/*!<# } #>!*/
    {
        /// <summary>
        /// Instance of <see cref="FileSystemInfo"/> corresponding to a file represented by this object.
        /// </summary>
        internal FileSystemInfo fileSystemInfo;

        /// <summary>
        /// Instance of <see cref="DavContext"/>.
        /// </summary>
        protected DavContext context;

        /// <summary>
        /// Encoded WebDAV relative path to the item.
        /// </summary>
        private readonly string path;

        /// <summary>
        /// User defined property values.
        /// </summary>
        private List<PropertyValue> propertyValues;
        //!<# if (Params.Class2) { #>

        /// <summary>
        /// Locks acquired on the item.
        /// </summary>
        private List<DateLockInfo> explicitLocks;
        //!<# } #>

        /// <summary>
        /// Initializes a new instance of the DavHierarchyItem class.
        /// </summary>
        /// <param name="fileSystemInfo">Corresponding <see cref="FileSystemInfo"/> to this file/folder.</param>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="path">Relative to WebDAV root folder path.</param>
        protected DavHierarchyItem(FileSystemInfo fileSystemInfo, DavContext context, string path)
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/: base(context)/*!<# } #>!*/
        {
            this.fileSystemInfo = fileSystemInfo;
            this.context = context;
            this.path = path;
        }

        //$<IHierarchyItemAsync.Name
        /// <summary>
        /// Gets name of the item.
        /// </summary>
        public string Name
        {
            get { return fileSystemInfo.Name; }
        }
        //$>

        //$<IHierarchyItemAsync.Created
        /// <summary>
        /// Gets date when the item was created in UTC.
        /// </summary>
        public DateTime Created
        {
            get { return fileSystemInfo.CreationTimeUtc; }
        }
        //$>

        //$<IHierarchyItemAsync.Modified
        /// <summary>
        /// Gets date when the item was last modified in UTC.
        /// </summary>
        public DateTime Modified
        {
            get { return fileSystemInfo.LastWriteTimeUtc; }
        }
        //$>

        //$<IHierarchyItemAsync.Path
        /// <summary>
        /// Gets path of the item where each part between slashes is encoded.
        /// </summary>
        public virtual string Path
        {
            get { return path; }
        }
        //$>

        /// <summary>
        /// Returns parent folder of the item.
        /// </summary>
        /// <returns>Parent folder.</returns>
        public async Task<DavFolder> GetParentAsync()
        {
            //Calculate local file system path of parent item.
            string parentLocalPath = fileSystemInfo.FullName.TrimEnd('\\');
            int index = parentLocalPath.LastIndexOf('\\');
            if (index <= context.RepositoryPath.Length - 1)
            {
                return null;
            }

            var directory = new DirectoryInfo(parentLocalPath.Remove(index));

            //Calculate WebDAV root relative parent path.
            string parentPath = Path.TrimEnd('/');
            int ind = parentPath.LastIndexOf('/');
            if (ind >= 0)
            {
                parentPath = parentPath.Substring(0, ind);
            }

            return (DavFolder) await context.GetHierarchyItemAsync(parentPath);
        }

        /// <summary>
        /// Creates a copy of this item with a new name in the destination folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="deep">Indicates whether to copy entire subtree.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task CopyToAsync(
            IItemCollectionAsync destFolder,
            string destName,
            bool deep,
            MultistatusException multistatus);

        /// <summary>
        /// Moves this item to the destination folder under a new name.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task MoveToAsync(IItemCollectionAsync destFolder, string destName, MultistatusException multistatus);

        /// <summary>
        /// Deletes this item.
        /// </summary>
        /// <param name="multistatus">If some items fail to delete but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task DeleteAsync(MultistatusException multistatus);

        //$<IHierarchyItemAsync.GetPropertiesAsync
        /// <summary>
        /// Retrieves user defined property values.
        /// </summary>
        /// <param name="names">Names of dead properties which values to retrieve.</param>
        /// <param name="allprop">Whether all properties shall be retrieved.</param>
        /// <returns>Property values.</returns>
        public async Task<IEnumerable<PropertyValue>> GetPropertiesAsync(IList<PropertyName> names, bool allprop)
        {
            if (allprop)
            {
                // get all properties
                return await getPropertyValuesAsync();
            }

            //Get requested property values.
            var propertyValues = await getPropertyValuesAsync();
            return propertyValues.Where(property => names.Contains(property.QualifiedName));
        }
        //$>

        //$<IHierarchyItemAsync.GetPropertyNamesAsync
        /// <summary>
        /// Retrieves names of all user defined properties.
        /// </summary>
        /// <returns>Property names.</returns>
        public async Task<IEnumerable<PropertyName>> GetPropertyNamesAsync()
        {
            var propertyValues = await getPropertyValuesAsync();
            return propertyValues.Select(p => p.QualifiedName);
        }
        //$>
        //!<# if (Params.Class2) { #>

        /// <summary>
        /// Check that if the item is locked then client has submitted correct lock token.
        /// </summary>
        public async Task RequireHasTokenAsync()
        {
            var allLocks = await getExplicitLocksAsync();
            if (allLocks.Count == 0)
            {
                return;
            }

            IList<string> clientLockTokens = context.Request.ClientLockTokens;
            if (allLocks.Any(l => clientLockTokens.Contains(l.LockToken)))
            {
                return;
            }

            throw new LockedException();
        }

        /// <summary>
        /// Ensure that there are no active locks on the item.
        /// </summary>
        /// <param name="skipShared">Whether shared locks shall be checked.</param>
        public async Task RequireUnlockedAsync(bool skipShared)
        {
            var explicitLocks = await getExplicitLocksAsync();
            if (skipShared && explicitLocks.Any(l => l.Level == LockLevel.Exclusive))
            {
                throw new LockedException();
            }

            if (!skipShared && explicitLocks.Any())
            {
                throw new LockedException();
            }
        }
        //!<# } #>

        //$<IHierarchyItemAsync.UpdatePropertiesAsync
        /// <summary>
        /// Saves dead property values to NTFS alternate data stream.
        /// </summary>
        /// <param name="setProps">Properties to be set.</param>
        /// <param name="delProps">Properties to be deleted.</param>
        /// <param name="multistatus">We don't use it here.</param>
        public virtual async Task UpdatePropertiesAsync(
            IList<PropertyValue> setProps,
            IList<PropertyName> delProps,
            MultistatusException multistatus)
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();

            //!<# } #>
            foreach (PropertyValue propToSet in setProps)
            {
                // Microsoft Mini-redirector may update file creation date, modification date and access time passing properties:
                // <Win32CreationTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:15:34 GMT</Win32CreationTime>
                // <Win32LastModifiedTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:36:24 GMT</Win32LastModifiedTime>
                // <Win32LastAccessTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:36:24 GMT</Win32LastAccessTime>
                // In this case update creation and modified date in your storage or do not save this properties at all, otherwise 
                // Windows Explorer will display creation and modification date from this props and it will differ from the values 
                // in the Created and Modified fields in your storage 
                if (propToSet.QualifiedName.Namespace == "urn:schemas-microsoft-com:")
                {
                    if (propToSet.QualifiedName.Name == "Win32CreationTime")
                    {
                        fileSystemInfo.CreationTimeUtc =
                            DateTime.Parse(propToSet.Value, new System.Globalization.CultureInfo("en-US")).ToUniversalTime();
                    }
                    else if (propToSet.QualifiedName.Name == "Win32LastModifiedTime")
                    {
                        fileSystemInfo.LastWriteTimeUtc =
                            DateTime.Parse(propToSet.Value, new System.Globalization.CultureInfo("en-US")).ToUniversalTime();
                    }
                }
                else
                {
                    var propertyValues = await getPropertyValuesAsync();
                    var existingProp = propertyValues.FirstOrDefault(p => p.QualifiedName == propToSet.QualifiedName);

                    if (existingProp != null)
                    {
                        existingProp.Value = propToSet.Value;
                    }
                    else
                    {
                        propertyValues.Add(propToSet);
                    }
                }
            }

            var propertyValuesForRemove = await getPropertyValuesAsync();
            propertyValuesForRemove.RemoveAll(prop => delProps.Contains(prop.QualifiedName));

            await context.FileOperationAsync(
                this,
                () => RewriteStreamAsync("Properties", propertyValues),
                Privilege.Write);

            // You should not update modification date/time here. Mac OS X Finder expects that properties update do not change the file modification date.
        }
        //$>
        //!<# if (Params.Class2) { #>

        //$<ILockAsync.GetActiveLocksAsync
        /// <summary>
        /// Retrieves non expired locks for this item.
        /// </summary>
        /// <returns>Enumerable with information about locks.</returns>
        public async Task<IEnumerable<LockInfo>> GetActiveLocksAsync()
        {
            //!<# if (Params.ACL) { #>
            RequireReadPrivilege();
            //!<# } #>
            var eplicitLocks = await getExplicitLocksAsync();
            return eplicitLocks.Select(
                    l => new LockInfo
                         {
                             IsDeep = l.IsDeep,
                             Level = l.Level,
                             Owner = l.ClientOwner,
                             LockRoot = l.LockRoot,
                             TimeOut = l.Expiration == DateTime.MaxValue ?
                                TimeSpan.MaxValue :
                                l.Expiration - DateTime.UtcNow,
                             Token = l.LockToken
                         });
        }
        //$>

        //$<ILockAsync.LockAsync

        /// <summary>
        /// Locks this item.
        /// </summary>
        /// <param name="level">Whether lock is share or exclusive</param>
        /// <param name="isDeep">Whether lock is deep.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <param name="owner">Owner of the lock as specified by client.</param> 
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<LockResult> LockAsync(LockLevel level, bool isDeep, TimeSpan? requestedTimeOut, string owner)
        {
            await RequireUnlockedAsync(level == LockLevel.Shared);

            string token = Guid.NewGuid().ToString();

            TimeSpan timeOut;
            if (!requestedTimeOut.HasValue || requestedTimeOut == TimeSpan.MaxValue)
            {
                // If timeout is absent or infinit timeout requested,
                // grant 5 minute lock.
                timeOut = TimeSpan.FromMinutes(5);
            }
            else
            {
                timeOut = requestedTimeOut.Value;
            }

            DateTime expiration = DateTime.UtcNow + timeOut;

            explicitLocks.Add(new DateLockInfo
                                  {
                                      Expiration = expiration,
                                      IsDeep = false,
                                      Level = level,
                                      LockRoot = Path,
                                      LockToken = token,
                                      ClientOwner = owner,
                                      //!<# if (Params.ACL) { #>
                                      OwnerUser = context.WindowsIdentity.Name,
                                      //!<# } #>
                                      TimeOut = timeOut
                                  });         
            await saveLocksAsync();
            return new LockResult(token, timeOut);
        }
        //$>

        //$<ILockAsync.RefreshLockAsync
        /// <summary>
        /// Updates lock timeout information on this item.
        /// </summary>
        /// <param name="token">Lock token.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<RefreshLockResult> RefreshLockAsync(string token, TimeSpan? requestedTimeOut)
        {
            var explicitLocks = await getExplicitLocksAsync();
            DateLockInfo li = explicitLocks.SingleOrDefault(l => l.LockToken == token);

            if (li == null)
            {
                throw new DavException("Lock can not be found.", DavStatus.PRECONDITION_FAILED);
            }

            if (requestedTimeOut.HasValue && requestedTimeOut != TimeSpan.MaxValue)
            {
                // Update timeout if it is specified and not Infinity.
                // Otherwise leave previous timeout.
                li.TimeOut = requestedTimeOut.Value;
            }

            if (!requestedTimeOut.HasValue)
                li.TimeOut = TimeSpan.FromMinutes(5);

            li.Expiration = DateTime.UtcNow + li.TimeOut;

            await saveLocksAsync();

            return new RefreshLockResult(li.Level, li.IsDeep, li.TimeOut, li.ClientOwner);
        }
        //$>

        //$<ILockAsync.UnlockAsync
        /// <summary>
        /// Removes lock with the specified token from this item.
        /// </summary>
        /// <param name="lockToken">Lock with this token should be removed from the item.</param>
        public async Task UnlockAsync(string lockToken)
        {
            var explicitLocks = await getExplicitLocksAsync();
            int i = explicitLocks.FindIndex(li => li.LockToken == lockToken);
            if (i >= 0)
            {
                //!<#if (Params.ACL) { #>
                var acl = File.GetAccessControl(
                    fileSystemInfo.FullName,
                    AccessControlSections.Access | AccessControlSections.Owner | AccessControlSections.Group);

                if (explicitLocks[i].OwnerUser == context.WindowsIdentity.Name || 
                    EffectivePermissions.GetEffectivePermissions(context.WindowsIdentity, acl) == FileSystemRights.FullControl)
                {
                    explicitLocks.RemoveAt(i);
                }
                else
                {
                    throw new NeedPrivilegesException(
                        "Only owner or Administrators can unlock resources",
                        Path,
                        Privilege.Unlock);
                }

                //!<# } #>
                //!<#if (!Params.ACL) { #>
                explicitLocks.RemoveAt(i);
                //!<# } #>
            }
            else
            {
                throw new DavException("The lock could not be found.", DavStatus.PRECONDITION_FAILED);
            }

            await saveLocksAsync();
        }
        //$>
        //!<# } #>
        //!<# if (Params.ACL) { #>

        //$<ICurrentUserPrincipalAsync.GetCurrentUserPrincipalAsync
        /// <summary>
        /// Returns instance of <see cref="IPrincipalAsync"/> which represents current user.
        /// </summary>
        /// <returns>Current user.</returns>
        /// <remarks>
        /// This method is usually called by the Engine when CalDAV/CardDAV client 
        /// is trying to discover current user URL.
        /// </remarks>
        public async Task<IPrincipalAsync> GetCurrentUserPrincipalAsync()
        {
            // Typically there is no need to load all user properties here, only current 
            // user ID (or name) is required to form the user URL: [DAVLocation]/acl/users/[UserID]

            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return AclFactory.GetPrincipalFromSid(context.WindowsIdentity.User.Value, context);
            //!<# } else if (Params.IsOwinAuthMode) { #>
            return new User(context, System.Web.HttpContext.Current.User.Identity.GetUserId());
            //!<# } else { #>
            return new User(context, context.UserName);
            //!<# } #>
        }
        //$>
        //!<# } #>
        //!<# if (Params.ACL && !(Params.CalDav || Params.CardDav)) { #>

        //$<IAclHierarchyItemAsync.SetOwnerAsync
        /// <summary>
        /// Sets <paramref name="principal"/> as new owner for this item.
        /// </summary>
        /// <param name="principal">New owner.</param>
        public async Task SetOwnerAsync(IPrincipalAsync principal)
        {
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
            context.FileOperation(
                this,
                () =>
                    {
                        var accessControl = File.GetAccessControl(fileSystemInfo.FullName, AccessControlSections.Owner);
                        accessControl.SetOwner(((PrincipalBase)principal).Sid);
                        File.SetAccessControl(fileSystemInfo.FullName, accessControl);
                    },
                Privilege.WriteProperties);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.GetOwnerAsync
        /// <summary>
        /// Retrieves user or group who is owner for this item.
        /// </summary>
        /// <returns>Owner for this item.</returns>
        public async Task<IPrincipalAsync> GetOwnerAsync()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return context.FileOperation(
                this,
                () =>
                    {
                        FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
                        return AclFactory.GetPrincipalFromSid(acl.GetOwner(typeof(SecurityIdentifier)).Value, context);
                    },
                Privilege.Read);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.SetGroupAsync
        /// <summary>
        /// File groups are not supported.
        /// </summary>
        public async Task SetGroupAsync(IPrincipalAsync principal)
        {
            throw new DavException("Group cannot be set", DavStatus.FORBIDDEN);
        }
        //$>

        //$<IAclHierarchyItemAsync.GetGroupAsync
        /// <summary>
        /// File groups are not supported.
        /// </summary>
        public async Task<IPrincipalAsync> GetGroupAsync()
        {
            return null;
        }
        //$>

        //$<IAclHierarchyItemAsync.GetSupportedPrivilegeSetAsync
        /// <summary>
        /// Retrieves list of all privileges (permissions) which can be set for the item.
        /// </summary>
        /// <returns>Enumerable with supported permissions.</returns>
        public async Task<IEnumerable<SupportedPrivilege>> GetSupportedPrivilegeSetAsync()
        {
            RequireReadPrivilege();
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return NtfsPermissions.GetPermissionsTree();
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.GetCurrentUserPrivilegeSetAsync
        /// <summary>
        /// Retrieves the exact set of permissions current user has for this file or folder.
        /// </summary>
        public async Task<IEnumerable<Privilege>> GetCurrentUserPrivilegeSetAsync()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            var acl = File.GetAccessControl(
                fileSystemInfo.FullName,
                AccessControlSections.Access | AccessControlSections.Owner | AccessControlSections.Group);

            return context.FileOperation(
                this,
                () => NtfsPermissions.ExpandPrivileges(
                    NtfsPermissions.MapFileSystemRights(
                        EffectivePermissions.GetEffectivePermissions(context.WindowsIdentity, acl))),
                Privilege.ReadAcl);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.GetAclAsync
        /// <summary>
        /// Retrieves access control list for this file or folder.
        /// </summary>
        /// <param name="propertyNames">Properties which will be retrieved from users/groups specified in
        /// access control list returned.</param>
        /// <returns>Enumerable with access control entries.</returns>
        public async Task<IEnumerable<ReadAce>> GetAclAsync(IList<PropertyName> propertyNames)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            AuthorizationRuleCollection rules = null;
            AuthorizationRuleCollection protectedRules = null;
            context.FileOperation(
                this,
                () =>
                    {
                        FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
                        rules = acl.GetAccessRules(true, false, typeof(SecurityIdentifier));
                        protectedRules = acl.GetAccessRules(false, true, typeof(SecurityIdentifier));
                    },
                Privilege.ReadAcl);

            IList<ReadAce> aceList = new List<ReadAce>();

            foreach (FileSystemAccessRule rule in rules)
            {
                var ace = new ReadAce();
                ace.Principal = AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context);
                if (ace.Principal == null)
                {
                    continue;
                }

                if (rule.AccessControlType == AccessControlType.Allow)
                {
                    ace.GrantPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }
                else
                {
                    ace.DenyPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }

                ace.IsProtected = false;
                aceList.Add(ace);
            }

            foreach (FileSystemAccessRule rule in protectedRules)
            {
                var ace = new ReadAce();
                ace.Principal = AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context);
                if (ace.Principal == null)
                {
                    context.Logger.LogDebug("Could not resolve principal with sid " + rule.IdentityReference.Value);
                    continue;
                }

                if (rule.AccessControlType == AccessControlType.Allow)
                {
                    ace.GrantPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }
                else
                {
                    ace.DenyPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }

                ace.IsProtected = true;
                ace.InheritedFrom = await GetParentAsync() ?? this;
                aceList.Add(ace);
            }

            return aceList;
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.SetAclAsync
        /// <summary>
        /// Sets access control list for this file/folder.
        /// </summary>
        /// <param name="aces">List of access control entries.</param>
        public async Task SetAclAsync(IList<WriteAce> aces)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
            var rules = acl.GetAccessRules(true, false, typeof(SecurityIdentifier));
            foreach (FileSystemAccessRule rule in rules)
            {
                //remove only those records we can resolve (in our domain/computer)
                if (AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context) != null)
                {
                   acl.RemoveAccessRule(rule);
                }
            }

            foreach (WriteAce ace in aces)
            {
                FileSystemRights grantRights = NtfsPermissions.MapPrivileges(ace.GrantPrivileges, context.Logger);
                var sid = ((PrincipalBase)ace.Principal).Sid;
                if (grantRights != 0)
                {
                    var grantRule = new FileSystemAccessRule(sid, grantRights, AccessControlType.Allow);
                    acl.AddAccessRule(grantRule);
                }

                FileSystemRights denyRights = NtfsPermissions.MapPrivileges(ace.DenyPrivileges, context.Logger);
                if (denyRights != 0)
                {
                    var denyRule = new FileSystemAccessRule(sid, denyRights, AccessControlType.Deny);
                    acl.AddAccessRule(denyRule);
                }
            }

            context.FileOperation(
                this,
                () => File.SetAccessControl(fileSystemInfo.FullName, acl),
                Privilege.WriteProperties);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.GetAclRestrictionsAsync
        /// <summary>
        /// Retrieves list of restrictions for access control entries.
        /// We don't support WebDAV inverted permissions.
        /// </summary>
        /// <returns>ACL restrictions.</returns>
        public async Task<AclRestriction> GetAclRestrictionsAsync()
        {
            return new AclRestriction { NoInvert = true };
        }
        //$>

        //$<IAclHierarchyItemAsync.GetInheritedAclSetAsync
        /// <summary>
        /// Gets all folders, from which this file/folder has inherited access control entries.
        /// </summary>
        /// <returns>Enumerable with files/folders from which this file/folder has inherited
        /// access control entries.</returns>
        public async Task<IEnumerable<IHierarchyItemAsync>> GetInheritedAclSetAsync()
        {
            var current = this;
            var parent = await current.GetParentAsync();
            List<IHierarchyItemAsync> parents = new List<IHierarchyItemAsync>();
            while (parent != null)
            {
                var accessControl = File.GetAccessControl(parent.fileSystemInfo.FullName);
                var acl = accessControl.GetAccessRules(true, false, typeof(SecurityIdentifier));
                if (acl.Count > 0)
                {
                    parents.Add(parent);
                }

                current = parent;
                parent = await current.GetParentAsync();
            }
            return parents;
        }
        //$>

        //$<IAclHierarchyItemAsync.GetPrincipalCollectionSetAsync
        /// <summary>
        /// Gets collections which contain principals.
        /// </summary>
        //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
        /// <remarks>
        /// We return one folder which contains users and the other folder which contains groups.
        /// We put them into different folders because webdav doesn't have strong distinction between users
        /// and groups, but Windows does, so we put users and groups into separate virtual folders so it is obvious
        /// what user wants to do (create user or group for example).
        /// </remarks>
        //!<# } #>
        /// <returns>Folders which contain users/groups.</returns>
        public async Task<IEnumerable<IPrincipalFolderAsync>> GetPrincipalCollectionSetAsync()
        {
            return new IPrincipalFolderAsync[]
                       {
                           //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
                           new UserFolder(context),
                           new GroupFolder(context)
                           //!<# } else { #>
                           new UsersFolder(context)
                           //!<# } #>
                       };
        }
        //$>

        //$<IAclHierarchyItemAsync.ResolveWellKnownPrincipalAsync
        /// <summary>
        /// Retrieves user or group which correspond to a well known principal
        /// (defined in <see cref="WellKnownPrincipal"/>.)
        /// </summary>
        /// <param name="wellKnownPrincipal">Well known principal type.</param>
        /// <returns>Instance of corresponding user/group or <c>null</c> if corresponding user/group
        /// is not supported.</returns>
        public async Task<IPrincipalAsync> ResolveWellKnownPrincipalAsync(WellKnownPrincipal wellKnownPrincipal)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            switch (wellKnownPrincipal)
            {
                case WellKnownPrincipal.All:
                    var sid1 = new SecurityIdentifier(WellKnownSidType.WorldSid, null);
                    return AclFactory.GetPrincipalFromSid(sid1.Value, context);
                case WellKnownPrincipal.Authenticated:
                    var sid2 = new SecurityIdentifier(WellKnownSidType.AuthenticatedUserSid, null);
                    return AclFactory.GetPrincipalFromSid(sid2.Value, context);
                case WellKnownPrincipal.Unauthenticated:
                    return context.AnonymousUser != null ? AclFactory.GetPrincipalFromSid(context.AnonymousUser.User.Value, context) : null;
                case WellKnownPrincipal.Self:
                    return null;
            }

            return null;
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItemAsync.GetItemsByPropertyAsync
        /// <summary>
        /// Find all resources in the subtree which have either Group or Owner (defined by <paramref name="matchBy"/>
        /// parameter) which identifies current user or group current user belongs to.
        /// For example, this report can return all of the resources in a collection
        /// hierarchy that are owned by the current user.
        /// We don't implement it here as all implementatino is in <see cref="DavFolder.GetItemsByProperty"/>.
        /// </summary>
        public virtual async Task<IEnumerable<IAclHierarchyItemAsync>> GetItemsByPropertyAsync(MatchBy matchBy, IList<PropertyName> props)
        {
            return new List<IAclHierarchyItemAsync>();
        }
        //$>

        /// <summary>
        /// Throws <see cref="NeedPrivilegesException"/> if current user doesn't have Read permission on this
        /// file/folder.
        /// </summary>
        protected void RequireReadPrivilege()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            var acl = File.GetAccessControl(
                fileSystemInfo.FullName,
                AccessControlSections.Access | AccessControlSections.Owner | AccessControlSections.Group);

            var rights = EffectivePermissions.GetEffectivePermissions(context.WindowsIdentity, acl);
            if ((rights & FileSystemRights.Read) != FileSystemRights.Read)
            {
                throw new NeedPrivilegesException("Missing read privilege", Path, Privilege.Read);
            }
            //!<# } #>
        }
        //!<# } #>

        //!<# if (Params.MicrosoftExtensions) { #>
        //$<IMsItemAsync.GetFileAttributesAsync
        /// <summary>
        /// Returns Windows file attributes (readonly, hidden etc.) for this file/folder.
        /// </summary>
        /// <returns>Windows file attributes.</returns>
        public async Task<FileAttributes> GetFileAttributesAsync()
        {
            //!<# if (Params.MarkMacOsFilesHidden) { #>
            if (Name.StartsWith("."))
            {
                return fileSystemInfo.Attributes | FileAttributes.Hidden;
            }
            //!<# } #>
            return fileSystemInfo.Attributes;
        }
        //$>

        //$<IMsItemAsync.SetFileAttributesAsync
        /// <summary>
        /// Sets Windows file attributes (readonly, hidden etc.) on this item.
        /// </summary>
        /// <param name="value">File attributes.</param>
        public async Task SetFileAttributesAsync(FileAttributes value)
        {
            File.SetAttributes(fileSystemInfo.FullName, value);
        }
        //$>
        //!<# } #>

        /// <summary>
        /// Retrieves full local path for this file/folder.
        /// </summary>
        /// <returns>Full local path.</returns>
        internal string GetFullPath()
        {
            return fileSystemInfo.FullName.TrimEnd('\\');
        }

        /// <summary>
        /// Gets NTFS stream with specified name and deserializes object of type <typeparamref name="T"/>
        /// from it.
        /// </summary>
        /// <typeparam name="T">Type of object serialized in stream.</typeparam>
        /// <param name="streamName">Name of NTFS stream stored as NTFS stream in this file/folder.</param>
        /// <param name="logger">Instance of <see cref="logger"/>.</param>
        /// <returns>Deserialized object.</returns>
        internal T GetStreamAndDeserialize<T>(string streamName, ILogger logger) where T : new()
        {
            return context.FileOperation(
                this,
                () => SerializationUtils.GetStreamAndDeserialize<T>(streamName, fileSystemInfo, logger),
                Privilege.Read);
        }

        //!<# if (Params.TargetFramework45OrLater) { #>

        /// <summary>
        /// Gets NTFS stream with specified name and deserializes object of type <typeparamref name="T"/>
        /// from it.
        /// </summary>
        /// <typeparam name="T">Type of object serialized in stream.</typeparam>
        /// <param name="streamName">Name of NTFS stream stored as NTFS stream in this file/folder.</param>
        /// <param name="logger">Instance of <see cref="logger"/>.</param>
        /// <returns>Deserialized object.</returns>
        internal async Task<T> GetStreamAndDeserializeAsync<T>(string streamName, ILogger logger) where T : new()
        {
            return await context.FileOperationAsync(
                this,
                () => SerializationUtils.GetStreamAndDeserializeAsync<T>(streamName, fileSystemInfo, logger),
                Privilege.Read);
        }

        //!<# } #>

        /// <summary>
        /// Serializes object <paramref name="o"/> and saves it into specified NTFS stream.
        /// </summary>
        /// <typeparam name="T">Type of object to be serialized.</typeparam>
        /// <param name="streamName">NTFS stream name.</param>
        /// <param name="o">Object to be serialized.</param>
        protected abstract Task RewriteStreamAsync<T>(string streamName, T o);

        /// <summary>
        /// Deletes NTFS stream with specified name.
        /// </summary>
        /// <param name="fileName">File name.</param>
        /// <param name="streamName">NTFS stream name.</param>
        protected abstract void DeleteStream(string fileName, string streamName);

        //!<# if (Params.Class2) { #>
        /// <summary>
        /// Retrieves non-expired locks acquired on this item.
        /// </summary>
        /// <returns>List of locks with their expiration dates.</returns>
        private async Task<List<DateLockInfo>> getExplicitLocksAsync()
        {
            if (explicitLocks == null)
            {
                explicitLocks = await GetStreamAndDeserializeAsync<List<DateLockInfo>>("Locks", context.Logger);
                explicitLocks.RemoveAll(li => li.Expiration <= DateTime.UtcNow);
                explicitLocks.ForEach(l => l.LockRoot = Path);
            }

            return explicitLocks;
        }    

        //!<# } #>   
     
        /// <summary>
        /// Retrieves list of user defined propeties for this item.
        /// </summary>
        /// <returns>List of user defined properties.</returns>
        private async Task<List<PropertyValue>> getPropertyValuesAsync()
        {
            if (propertyValues == null)
            {
                // Deserialize property values from alternate data stream.
                propertyValues = await GetStreamAndDeserializeAsync<List<PropertyValue>>("Properties", context.Logger);
            }

            return propertyValues;
        }
       
        //!<# if (Params.Class2) { #>
        /// <summary>
        /// Saves locks acquired on this file/folder in NTFS stream.
        /// </summary>
        private async Task saveLocksAsync()
        {
            var explicitLocks = await getExplicitLocksAsync();
            if (explicitLocks.Count > 0)
            {
                await RewriteStreamAsync("Locks", explicitLocks);
            }
            else
            {
                DeleteStream(fileSystemInfo.FullName, "Locks");
            }
        }
        //!<# } #>
    }
}
