<#@ template language="C#" #>
<#@ samplevars Processor="DirectiveProcessor" #>
<# if (Params.CalDav || Params.CardDav)  return string.Empty; #>
<# if (Params.Storage != StorageType.Db)  return string.Empty; #>

USE [master]
GO

CREATE DATABASE <#=Params.DBName#>
GO

USE [<#=Params.DBName#>]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[Item](
     [ItemId] [uniqueidentifier] NOT NULL
    ,[ItemType] [int] NOT NULL
    ,[ParentItemId] [uniqueidentifier] NOT NULL
    ,[Name] [nvarchar](255) NOT NULL
    ,[Created] [datetime] NOT NULL
    ,[Modified] [datetime] NOT NULL
    ,[ContentType] [nvarchar](255) NULL
    ,[Content] varbinary(max) NULL
    ,[SerialNumber] [int] NULL
    <# if (Params.ResumableUpload || Params.IEUpload) { #>
    ,[TotalContentLength] bigint NOT NULL
    ,[LastChunkSaved] [datetime] NULL
    <# } #>
    <# if (Params.MicrosoftExtensions) { #>,[FileAttributes] [int] NOT NULL <# } #>
    <# if (Params.DeltaV) { #>,[CreatorDisplayName] [nvarchar](255) NOT NULL
    ,[Comment] [nvarchar](2000) NOT NULL
    ,[CheckedOut] [bit] NULL
    ,[AutoVersion] [int] NOT NULL
    ,[AutoCheckedOut] [bit] NOT NULL
    <# } #>CONSTRAINT [PK_Item] PRIMARY KEY CLUSTERED([ItemId] ASC)
)
GO

CREATE UNIQUE NONCLUSTERED INDEX [IX_Item] ON [dbo].[Item] 
(
    [ParentItemId] ASC,
    [Name] ASC
)
GO

CREATE TABLE [dbo].[Property](
    [PropertyId] [uniqueidentifier] NOT NULL DEFAULT NEWID(),
    [ItemId] [uniqueidentifier] NOT NULL,
    [Name] [nvarchar](255) NOT NULL,
    [Namespace] [nvarchar](100) NOT NULL,
    [PropVal] [nvarchar](255) NOT NULL,
    CONSTRAINT [PK_Property] PRIMARY KEY CLUSTERED ([PropertyId] ASC)
)
GO

CREATE UNIQUE NONCLUSTERED INDEX [IX_Property] ON [dbo].[Property]
(
    [Name] ASC,
    [Namespace] ASC,
    [ItemId] ASC
)
GO

<# if (Params.Class2) { #>
CREATE TABLE [dbo].[Lock](
    [Token] [nvarchar](36) NOT NULL,
    [ItemId] [uniqueidentifier] NOT NULL,
    [Shared] [bit] NOT NULL,
    [Deep] [bit] NOT NULL,
    [Expires] [datetime] NOT NULL,
    [Owner] [nvarchar](255) NOT NULL,
    CONSTRAINT [PK_Lock] PRIMARY KEY CLUSTERED ([Token] ASC)
)
GO
<# } #>

<# if (Params.DeltaV) { #>
CREATE TABLE [dbo].[Version](
    [VersionId] [uniqueidentifier] NOT NULL,
    [ItemId] [uniqueidentifier] NOT NULL,
    [VersionNumber] [int] NOT NULL,
    [Name] [nvarchar](255) NOT NULL,
    [Comment] [nvarchar](2000) NOT NULL,
    [CreatorDisplayName] [nvarchar](255) NOT NULL,
    [ContentType] [nvarchar](255) NULL,
    [Content] varbinary(max),
    [Created] [datetime] NOT NULL,
    [SerialNumber] [int] NOT NULL
    CONSTRAINT [PK_Version] PRIMARY KEY CLUSTERED ([VersionId] ASC)
)
GO

CREATE UNIQUE NONCLUSTERED INDEX [IX_Version] ON [dbo].[Version] 
(
    [ItemId] ASC,
    [VersionNumber] ASC
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[VersionProperty](
    [VersionPropertyId] [uniqueidentifier] NOT NULL DEFAULT NEWID(),
    [VersionId] [uniqueidentifier] NOT NULL,
    [Name] [nvarchar](255) NOT NULL,
    [Namespace] [nvarchar](100) NOT NULL,
    [PropVal] [nvarchar](255) NOT NULL,
    CONSTRAINT [PK_VersionProperty] PRIMARY KEY CLUSTERED ([VersionPropertyId] ASC)
)

GO
CREATE UNIQUE NONCLUSTERED INDEX [IX_PropertyVersion] ON [dbo].[VersionProperty] 
(
    [VersionId] ASC,
    [Name] ASC,
    [Namespace] ASC
)

GO

<# } #>

DECLARE @ParentID [uniqueidentifier]
SELECT @ParentID='00000000-0000-0000-0000-000000000001'

INSERT INTO [dbo].[Item] (ItemId, Name, Created, Modified, ParentItemId, ItemType<# if (Params.ResumableUpload || Params.IEUpload) { #>, TotalContentLength<# } #><# if (Params.MicrosoftExtensions) { #>, FileAttributes<# } #><# if (Params.DeltaV) { #>, CreatorDisplayName, Comment, AutoVersion, AutoCheckedOut<# } #>)
VALUES(@ParentID, N'Root', GETUTCDATE(), GETUTCDATE(), '00000000-0000-0000-0000-000000000000', 3<# if (Params.ResumableUpload || Params.IEUpload) { #>, 0<# } #><# if (Params.MicrosoftExtensions) { #>, <#=(int)System.IO.FileAttributes.Directory#><# } #><# if (Params.DeltaV) { #>, '', '', 0, 0<# } #>);

<# if (!String.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
DECLARE @DavLocationID [uniqueidentifier]
<# string[] folders = Params.DavUrlPrefix.Trim(new char[]{'/', ' '}).Split(new char[]{'/'});
foreach (string folder in folders)
{ #>
SET @DavLocationID = NEWID()
INSERT INTO [dbo].[Item] (ItemId, Name, Created, Modified, ParentItemId, ItemType<# if (Params.ResumableUpload || Params.IEUpload) { #>, TotalContentLength<# } #><# if (Params.MicrosoftExtensions) { #>, FileAttributes<# } #><# if (Params.DeltaV) { #>, CreatorDisplayName, Comment, AutoVersion, AutoCheckedOut<# } #>)
VALUES(@DavLocationID, N'<#=folder#>', GETUTCDATE(), GETUTCDATE(), @ParentID, 3<# if (Params.ResumableUpload || Params.IEUpload) { #>, 0<# } #><# if (Params.MicrosoftExtensions) { #>, <#=(int)System.IO.FileAttributes.Directory#><# } #><# if (Params.DeltaV) { #>, '', '', 0, 0<# } #>);
SET @ParentID = @DavLocationID
<# } #>
<# } #>

<# if (Params.PopulateWithSampleData) {
   var folders = new System.Collections.Generic.Dictionary<string, Guid>();
   foreach (string file in  ITHit.WebDav.Server.SampleGenerator4_5_2657.FileEnumerator.EnumerateDirectory(Params.SampleDataPath, null, true))
   {
       string parentId = folders.ContainsKey(System.IO.Path.GetDirectoryName(file)) ? "'"+ folders[System.IO.Path.GetDirectoryName(file)] +"'" : "@ParentID";
       if (System.IO.Directory.Exists(file))
       {
         folders.Add(file, Guid.NewGuid()); 
         string name = new System.IO.DirectoryInfo(file).Name;#>
INSERT INTO [dbo].[Item] (ItemId, Name, Created, Modified, ParentItemId, ItemType<# if (Params.ResumableUpload || Params.IEUpload) { #>, TotalContentLength<# } #><# if (Params.MicrosoftExtensions) { #>, FileAttributes<# } #><# if (Params.DeltaV) { #>, CreatorDisplayName, Comment, AutoVersion, AutoCheckedOut<# } #>)
VALUES ('<#=folders[file]#>', N'<#=name#>', GETUTCDATE(), GETUTCDATE(), <#=parentId#>, 3<# if (Params.ResumableUpload || Params.IEUpload) { #>, 0<# } #><# if (Params.MicrosoftExtensions) { #>, <#=(int)System.IO.FileAttributes.Directory#><# } #><# if (Params.DeltaV) { #>, '', '', 0, 0<# } #>);

       <#}
       else
       {
         string content = "0x" + BitConverter.ToString(System.IO.File.ReadAllBytes(file)).Replace("-","");
         string name = new System.IO.DirectoryInfo(file).Name; #>
INSERT INTO [dbo].[Item] (ItemId, Name, Created, Modified, ParentItemId, ItemType<# if (Params.ResumableUpload || Params.IEUpload) { #>, TotalContentLength<# } #><# if (Params.MicrosoftExtensions) { #>, FileAttributes<# } #><# if (Params.DeltaV) { #>, CreatorDisplayName, Comment, AutoVersion, AutoCheckedOut<# } #>, Content)
VALUES (NEWID(), N'<#=name#>', GETUTCDATE(), GETUTCDATE(), <#=parentId#>, 2<# if (Params.ResumableUpload || Params.IEUpload) { #>, 0<# } #><# if (Params.MicrosoftExtensions) { #>, <#=(int)System.IO.FileAttributes.Normal#><# } #><# if (Params.DeltaV) { #>, '', '', 0, 0<# } #>, <#=content#>);

      <#}
    }#>
GO

	<# if (Params.EnableVersionControlOnInitialFiles) { #>
INSERT INTO Version
SELECT NEWID(), ItemId, 1, Name, '', '', ContentType, [Content], Created, 0
FROM Item WHERE (ItemType = 2)
GO

UPDATE Item SET AutoVersion=<#=(int)Params.AutoVersionMode #> WHERE (ItemType = 2)
GO
	<# } #>

<# } #>

ALTER TABLE [dbo].[Property]  WITH CHECK ADD  CONSTRAINT [FK_Property_Item1] FOREIGN KEY([ItemId])
REFERENCES [dbo].[Item] ([ItemId]) ON DELETE CASCADE
GO

<# if (Params.Class2) { #>
ALTER TABLE [dbo].[Lock]  WITH CHECK ADD  CONSTRAINT [FK_Lock_Item] FOREIGN KEY([ItemId])
REFERENCES [dbo].[Item] ([ItemId]) ON DELETE CASCADE
GO
<# } #>

<# if (Params.DeltaV) { #>
ALTER TABLE [dbo].[Version]  WITH CHECK ADD  CONSTRAINT [FK_Version_Item1] FOREIGN KEY([ItemId])
REFERENCES [dbo].[Item] ([ItemId]) ON DELETE CASCADE
GO

ALTER TABLE [dbo].[VersionProperty]  WITH CHECK ADD  CONSTRAINT [FK_PropertyVersion_Version] FOREIGN KEY([VersionId])
REFERENCES [dbo].[Version] ([VersionId]) ON DELETE CASCADE
GO
<# } #>

<# if (Params.DASL) { #>
-- To enable full-text search uncomment and execute the code below and uncoment code in Search method in DavFolder.cs 
-- To enable full-text search for .docx, .pptx, .xlsx files install the following software on your MS SQL machine:
-- Microsoft Office 2010 Filter Packs
-- http://www.microsoft.com/en-us/download/details.aspx?id=17062
-- And Service Pack 2 for Microsoft Office 2010 Filter Pack 64-Bit Edition
-- http://www.microsoft.com/en-us/download/details.aspx?id=39668
-- Or Service Pack 2 for Microsoft Office 2010 Filter Pack 32-Bit Edition
-- http://www.microsoft.com/en-us/download/details.aspx?id=39671

--CREATE FULLTEXT CATALOG FileCatalog AS DEFAULT;
--GO

--CREATE FULLTEXT INDEX ON [WebDav].[dbo].[Item]
--([Content] TYPE COLUMN [Name])
--KEY INDEX [PK_Item]
--ON FileCatalog
--WITH CHANGE_TRACKING AUTO;
--GO

-- End of block
<# } #>
